/***************************************************************************/
/*                                                                         */
/*                              FKISS function parser                      */
/*                                                                         */
/***************************************************************************/

#ifndef __FKISS_IO_H
#define __FKISS_IO_H

#include "BOOL.h"

#include "KISSTypes.h"
#include "FKISSTypes.h"

/***************************************************************************/

#define FKS_P_Success      0

#define FKS_P_BadObj       1
#define FKS_P_ObjExp       2
#define FKS_P_CelExp       3
#define FKS_P_BadCel       4
#define FKS_P_ItemOrCel    5
#define FKS_P_BadSet       6
#define FKS_P_BadAlarm     7
#define FKS_P_BadPalette   8
#define FKS_P_BadSyntax    9
#define FKS_P_BadFunc     10
#define FKS_P_BadFile     11
#define FKS_P_BadInt      12
#define FKS_P_BadParams   13
#define FKS_P_BadString   14
#define FKS_P_BadVar      15
#define FKS_P_BadLabel    16
#define FKS_P_Ignored    -17
#define FKS_P_MissFunc   -18
#define FKS_P_UnclosedIf -19
#define FKS_P_OrphanIf   -20
#define FKS_P_NestedIf   -21
#define FKS_P_BadKeyName  22
#define FKS_P_DupHandler  23
#define FKS_P_FileTooLong 24

/***************************************************************************/

extern BOOL IsFKISSFunction(char *s);

/***************************************************************************/

extern void process_fkiss_command_line(char *fkiss);

/***************************************************************************/

typedef BOOL (*action_list_function)(FKISS_ACTION_LIST *action,void *handle);
typedef BOOL (*event_list_function)(FKISS_EVENT_LIST *event,void *handle);

/***************************************************************************/
/*
 * This routine works as follows:
 *   Two functions are provided, one applying to actions and one to events.
 *   The flags word specifies whether the function is called for every action
 *   or just the first in any one list, ditto for events.
 *   The handle is a parameter to the provided functions
 */

#define FKISS_APPLY_ALLEVENTS  (1u<<0)
#define FKISS_APPLY_ALLACTIONS (1u<<1)

/***************************************************************************/

extern BOOL fkiss_apply_to_all(int flags,
                               event_list_function  eventfunc,
                               action_list_function actionfunc,
                               void *handle);

/***************************************************************************/

extern BOOL fkiss_load_completed(void);

/***************************************************************************/

extern void fkiss_clear_doll(void);

#endif
