/***************************************************************************/
/*                                                                         */
/*                     Types used by FKISS Parser and printer              */
/*                                                                         */
/***************************************************************************/

#ifndef __FKPARAMTYPES_H
#define __FKPARAMTYPES_H

#include "KISSTypes.h"
#include "FKISSTypes.h"

typedef enum __paramtp { fks_p_celgrp     =  1,
                         fks_p_object     =  2,
                         fks_p_objorvar   =  3,
                         fks_p_celgrpobj  =  4,
                         fks_p_cogv       =  5,
                         fks_p_grp        =  6,
                         fks_p_string     =  7,
                         fks_p_file       =  8,
                         fks_p_constant   =  9,
                         fks_p_variable   = 10,
                         fks_p_intval     = 11,
                         fks_p_keyid      = 12,
                         fks_p_idtag      = 13 } FKISS_PARAMETER_TYPE;

typedef union __param {
                        FKISS_INSTANCE_TYPE  instance;
                        FKISS_INTEGERVALUE   intval;
                        char                 *string;
                        char                 *filename;
                        int                  keyid;
                        FKISS_VARIABLE_LIST  variable;
                        char                 *idtag; } FKISS_PARAMETER;

/*
 * How to encode the fkiss version number in the function definition array.
 */

#define FKISS_VERSION_BITS  4
#define FKISS_VERSION_MASK  ((1<<FKISS_VERSION_BITS)-1)
#define FKISS_VERSION_SHIFT (32-FKISS_VERSION_BITS)
#define FVERS(n)            (n<<FKISS_VERSION_SHIFT)

/*
 * Get the external FKISS version number for a function.
 */

#define FKISS_VERSION(f)    (((f.params>>FKISS_VERSION_SHIFT) & FKISS_VERSION_MASK) \
                            + KISS_V_FKISS)

#define FKISS_ERROR_BITS    3
#define FKISS_ERROR_MASK    ((1<<FKISS_ERROR_BITS)-1)
#define FKISS_ERROR_SHIFT   (32-FKISS_VERSION_BITS-FKISS_ERROR_BITS)
#define FERROR(f)           (f<<FKISS_ERROR_SHIFT)
#define FKISS_ERROR(f)      ((f.params>>FKISS_ERROR_SHIFT) & FKISS_ERROR_MASK)

#define FKISS_MISSPELLING   1
#define FKISS_SYNONYM       2
#define FKISS_UNSUPPORTED   3
#define FKISS_NOTPORTABLE   4

#define BITSPERPARAM 4
#define PTYPEIS(p,t)     ((t)<<((p)*BITSPERPARAM))
#define P_CELGRP(n)      PTYPEIS(n,fks_p_celgrp)
#define P_OBJECT(n)      PTYPEIS(n,fks_p_object)
#define P_OBJVAR(n)      PTYPEIS(n,fks_p_objorvar)
#define P_COG(n)         PTYPEIS(n,fks_p_celgrpobj)
#define P_COGV(n)        PTYPEIS(n,fks_p_cogv)
#define P_GROUP(n)       PTYPEIS(n,fks_p_grp)
#define P_STRING(n)      PTYPEIS(n,fks_p_string)
#define P_FILE(n)        PTYPEIS(n,fks_p_file)
#define P_CONSTANT(n)    PTYPEIS(n,fks_p_constant)
#define P_CONSTVAL(n)    PTYPEIS(n,fks_p_constvalue)
#define P_VARIABLE(n)    PTYPEIS(n,fks_p_variable)
#define P_INTVAL(n)      PTYPEIS(n,fks_p_intval)
#define P_KEYID(n)       PTYPEIS(n,fks_p_keyid)
#define P_IDTAG(n)       PTYPEIS(n,fks_p_idtag)

#define MAXPARAMCOUNT (((8*sizeof(int))-FKISS_VERSION_BITS)/BITSPERPARAM)
#define PARAMMASK  ((1<<BITSPERPARAM)-1)
#define PARAMETERTYPE(p,n) (((p)>>(BITSPERPARAM*(n))) & PARAMMASK)

typedef int (*fkiss_event_parser)(int fno,FKISS_EVENT_DETAILS *event);

typedef struct { char               *funcname;
                 int                fid;
                 int                params;
                 fkiss_event_parser eventfn;
               } FKISS_LOAD_STR;

#endif
