/***************************************************************************/
/*                                                                         */
/*                              Types for KISS                             */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

#ifndef __KISSTYPES_H
#define __KISSTYPES_H

#include "BOOL.h"

#define MAXDOSNAMELEN 12

#define MAXLEAFNAMELEN       64
#define MAXSPRITENAMELEN     12
#define MAX_INPUT_BUFFER_LEN 512
#define MAX_LINE_LENGTH      256

#define MAX_SETS 10
#define MAX_CELS 512

#define MAX_DOLL_NAME_LEN 56

#define MAX_CEL_X      768
#define MAX_CEL_Y      480
#define PAL_MAX_CPG    256
#define PAL_MAX_GRP    10

#define INRANGE(x,a,b) (((x)>=(a)) && ((x)<=(b)))
#define MAX(x,y) (((x)<(y))?(y):(x))
#define MIN(x,y) (((x)<(y))?(x):(y))

typedef unsigned char BYTE;

typedef struct __pal {
                       char         *name;
                       int           colspergroup;
                       struct __pal *next;
                     } KISS_PALETTE_TYPE;

typedef KISS_PALETTE_TYPE *KISS_PALETTE_LIST;

typedef struct __crdstr { int x; int y; } COORD_PAIR;

typedef struct __obj { COORD_PAIR    startpos[MAX_SETS];
                       COORD_PAIR    position[MAX_SETS];
                       COORD_PAIR    size;
                       int           setmask;
                       struct __cel *first_cel;
                       int           mark;
                       int           fix;
                       int           dragcount;
                       struct __obj *parent;
                       struct __obj *child;
                       struct __obj *sibling;
                       int           flags;
                       struct __obj *next;
                     } KISS_OBJECT_TYPE;

typedef KISS_OBJECT_TYPE *KISS_OBJECT_LIST;

typedef struct __cel { COORD_PAIR         offset;
                       COORD_PAIR         size;
                       int                bpp;
                       KISS_OBJECT_LIST   objectid;
                       struct __cel       *next_cel;
                       int                setmask;
                       int                flags;
                       int                fix;
                       int                dragcount;
                       int                palette;
                       int                starttransp;
                       char               *celname;
                       struct __cel       *next;
                       struct __cel       *prev;
                       struct __cel       *duplicate;
                      } KISS_CEL_TYPE;

/* Cel flags */

#define CEL_K_UnMapped      (1u<<0)
#define CEL_K_Ghost         (1u<<1)
#define CEL_K_Cherry        (1u<<2)
#define CEL_K_PriorUnMap    (1u<<3)
#define CEL_K_StartGhosted  (1u<<4)
#define CEL_K_OverrideXOff  (1u<<5)
#define CEL_K_OverrideYOff  (1u<<6)
#define CEL_K_StartUnmapped (1u<<7)

#define CEL_K_GS2           (1u<<16)
#define CEL_K_MovePending   (1u<<17)
#define CEL_K_OLEFile       (1u<<18)

#define CEL_K_StartFlags  (CEL_K_GS2 | CEL_K_Cherry)

typedef KISS_CEL_TYPE *KISS_CEL_LIST;

#define MAXFIXVAL 32767

#define MAX_KISS_VERSION  8
#define KISS_V_GS1        0
#define KISS_V_GS2        1
#define KISS_V_GS1PLUS    2
#define KISS_V_GS2PLUS    3
#define KISS_V_FKISS      4
#define KISS_V_FKISS2     5
#define KISS_V_FKISS3     6
#define KISS_V_FKISS4     7

/*  Structure to define GS rank limits */

typedef struct {
                 COORD_PAIR maxSize;
                 int        maxColours;
                 int        maxCels;
                 int        maxObjects;
               } KISS_RANK_LIMITS_TYPE;

#define MAX_KISS_COLOURMODEL   6

#define KISS_V_NOMODEL         0
#define KISS_V_GS1PALETTE      1
#define KISS_V_GS2PALETTE      2
#define KISS_V_EXTENDEDPALETTE 3
#define KISS_V_CHERRY          4
#define KISS_V_HYBRID          5

#endif
