/***************************************************************************/
/*                                                                         */
/*                                KISS Doll data                           */
/*                                                                         */
/***************************************************************************/

#ifndef __KISS_DATA_H
#define __KISS_DATA_H

#include "BOOL.h"
#include "KISSTypes.h"
#include "FKISSTypes.h"
#include "lzh.h"

/* bounds checking states.
 * Constants are interpreted as:
 *
 *  All        : Full bounds checking on both user drags and FKISS
 *  FKISSOnly  : Full bounds checking on FKISS, none on user drags
 *  Visual     : "Visible" bounds (i.e. consider map/unmap) on user drags
 *               and FKISS
 *  Mixed      : Treat as Visual for FKISS, none on user drags.
 *
 * They can thus be grouped as:
 *  For user drags: All, FKISSOnly+Mixed, Visual
 *  For FKISS: All+FKISSOnly, Visual+Mixed
 */

#define BOUNDS_K_Off       0
#define BOUNDS_K_Visual    1
#define BOUNDS_K_Full      2

#define BOUNDS_K_TypeMask  3
#define BOUNDS_K_FKShift   2

extern BOOL boundsChecking;

#define BOUNDS_DRAG  (boundsChecking & BOUNDS_K_TypeMask)
#define BOUNDS_FKISS ((boundsChecking >> BOUNDS_K_FKShift) & BOUNDS_K_TypeMask)
#define BOUNDS(d,f)  ((d & BOUNDS_K_TypeMask) | ((f & BOUNDS_K_TypeMask)<<BOUNDS_K_FKShift))

extern BOOL case_sensitive_fkiss;

extern const int default_border;
extern const COORD_PAIR default_imagesize;

extern int border_colour;

extern COORD_PAIR imagesize;
extern COORD_PAIR requestedSize; /* The size in the cnf file */

extern KISS_PALETTE_LIST KISS_palettes;
extern KISS_OBJECT_LIST KISS_objects;
extern KISS_CEL_LIST    KISS_cels;

extern int KISS_palette_group[MAX_SETS];

extern int CelCount, ObjectCount, PaletteCount, ActiveSetsMask;
extern int MaxFixValue;

extern BOOL CherryKISS,
            ExtendedPalette;

extern FKISS_EVENT_LIST fkiss_misc_list,
                        fkiss_alarm_list,
                        fkiss_mouse_list,
                        fkiss_collision_list,
                        fkiss_label_list,
                        fkiss_key_list,
                        fkiss_pointer_list;

extern int fkiss_flags;

extern FKISS_VARIABLE_LIST fkiss_variables;

extern FKISS_GROUP_LIST fkiss_groups;

extern int KISS_Version,
           KISS_ColourModel;

extern const KISS_RANK_LIMITS_TYPE rankLimits[KISS_V_GS2PLUS];

/*
 * Stuff to support lzh extraction
 */

extern char *lzhArchiveName;

extern LZH_CONTENT_LIST lzhArchiveContents;

extern BOOL syntax_only,
            show_progress,
            remove_working_directory;

extern char *working_directory;

#endif
