/***************************************************************************/
/*                                                                         */
/*                        Cel group creation & control                     */
/*                                                                         */
/***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utilities.h"

#include "BOOL.h"

#include "KISSTypes.h"
#include "FKISSTypes.h"
#include "KISS_data.h"

#define NULL_FRAME -1

/***************************************************************************/

FKISS_GROUP_LIST celgroup_findOrCreateGroup(char *name,BOOL create)
{
  FKISS_GROUP_LIST group=fkiss_groups, prev=NULL, newgrp;
  int compval;

  while ( group )
  {
    compval = struncmp(group->name,name,MaxIdentifierLen);
    if ( compval >= 0 )
      break;
    prev  = group;
    group = group->nextGroup;
  }

  if ( (compval > 0) || !group )
  {
    if ( !create )
      return NULL;

    newgrp = safe_alloc(sizeof(FKISS_GROUP_TYPE));
    memset(newgrp,0,sizeof(FKISS_GROUP_TYPE));

    if ( prev )
      prev->nextGroup = newgrp;
    else fkiss_groups = newgrp;

    newgrp->nextGroup = group;

    newgrp->maxFrame = -1;
    strncpy(newgrp->name,name,MaxIdentifierLen);

    group = newgrp;
  }
  return group;
}

/***************************************************************************/

#define celgroup_findGroup(group) celgroup_findOrCreateGroup(group,FALSE)

/***************************************************************************/

FKISS_FRAME_LIST celgroup_findOrCreateFrame(FKISS_GROUP_LIST group,
                                            int              frameno,
                                            BOOL             create)
{
  FKISS_FRAME_LIST frame=group->frames,prev=NULL, newframe;

  if ( frameno == NULL_FRAME )
    return NULL;

  while ( frame && (frameno > frame->frameNo) )
  {
    prev  = frame;
    frame = frame->nextFrame;
  }

  if ( !frame || (frameno < frame->frameNo) )
  {
    if ( !create )
      return NULL;

    newframe = safe_alloc(sizeof(FKISS_FRAME_TYPE));
    memset(newframe,0,sizeof(FKISS_FRAME_TYPE));
    if ( prev )
      prev->nextFrame = newframe;
    else group->frames = newframe;

    newframe->nextFrame = frame;

    newframe->frameNo = frameno;
    group->maxFrame   = MAX(group->maxFrame,frameno);
    frame             = newframe;
  }
  return frame;
}

#define celgroup_findFrame(grp,f) \
   celgroup_findOrCreateFrame(grp,f,FALSE)

/***************************************************************************/

static BOOL celInArray(KISS_CEL_LIST cel,CEL_ARRAY_PTR celArray)
{
  int celno=0;

  for ( celno = 0; celno < celArray->numCel; celno++ )
    if ( cel == celArray->cels[celno] )
      return TRUE;

  return FALSE;
}

/***************************************************************************/

#define CEL_ARRAY_INC 16

static void addCelToArray(KISS_CEL_LIST cel,CEL_ARRAY_PTR celArray)
{
  if ( !celInArray(cel,celArray) )
  {
    if ( celArray->numCel >= celArray->capacity )
    {
      int newcap = celArray->capacity + CEL_ARRAY_INC;

      celArray->cels = realloc(celArray->cels,newcap * sizeof(KISS_CEL_LIST));
      if ( !celArray->cels )
      {
        fputs("Out of memory\n",stderr);
        exit(1);
      }

      celArray->capacity = newcap;
    }
    celArray->cels[celArray->numCel] = cel;
    (celArray->numCel)++;
  }
}

/***************************************************************************/

void celgroup_addCelToGroupId(FKISS_GROUP_LIST group,
                              int              frameno,
                              KISS_CEL_LIST    cel)
{
  FKISS_FRAME_LIST frame;

  addCelToArray(cel,(&group->cels));

  if ( frameno != NULL_FRAME )
  {
    frame = celgroup_findOrCreateFrame(group,frameno,TRUE);
    addCelToArray(cel,&(frame->cels));
  }

}

/***************************************************************************/

void celgroup_addCelToGroup(char         *name,
                            int           frameno,
                            KISS_CEL_LIST cel)
{
  FKISS_GROUP_LIST group;

  group = celgroup_findOrCreateGroup(name,TRUE);
  celgroup_addCelToGroupId(group,frameno,cel);
}

/***************************************************************************/

BOOL celgroup_isMember(FKISS_GROUP_LIST grp,KISS_CEL_LIST cel)
{
  return celInArray(cel,&(grp->cels));
}

/***************************************************************************/

static void disposeFrames(FKISS_FRAME_LIST *frame)
{
  FKISS_FRAME_LIST next;

  while ( *frame )
  {
    next   = *frame;
    *frame = next->nextFrame;
    free(next->cels.cels);
    free(next);
  }
}

/***************************************************************************/

void celgroup_dispose(void)
{
  FKISS_GROUP_LIST group;

  while ( fkiss_groups )
  {
    group        = fkiss_groups;
    fkiss_groups = group->nextGroup;
    disposeFrames(&(group->frames));
    free(group->cels.cels);
    free(group);
  }
}
