/*----------------------------------------------------------------------*/
/*                                                                      */
/*                         Mini LZH handler                             */
/*                    Derived from LHarc for UNIX                       */
/*                                                                      */
/*            Depends on 'lharc' command being in CLI path              */
/*                                                                      */
/*----------------------------------------------------------------------*/

#ifndef __LZH_H
#define __LZH_H

#include <stdio.h>

/*----------------------------------------------------------------------*/
/*
 * Substitute various parameters into an OS command and then execute
 * the command.
 */

extern void lzh_processCommand(const char *format,
                               const char *archiveName,
                               const char *filename);

/*----------------------------------------------------------------------*/
/*
 * Scan an lzh archive and produce a list of filenames included in the
 * archive.
 *
 * Returns: status value.
 */

#define LZH_NOTFOUND -2
#define LZH_INVALID  -1
#define LZH_OK        0

typedef struct lzhcont {
                          char           *filename;
                          struct lzhcont *next;
                        } LZH_CONTENT_TYPE;

typedef LZH_CONTENT_TYPE *LZH_CONTENT_LIST;

extern int lzh_scanFile(const char *filename,LZH_CONTENT_LIST *contents);

/*----------------------------------------------------------------------*/

extern LZH_CONTENT_LIST lzh_findCNF(LZH_CONTENT_LIST contents);

/*----------------------------------------------------------------------*/

extern LZH_CONTENT_LIST lzh_findFile(LZH_CONTENT_LIST contents,
                                     const char       *filename);

/*----------------------------------------------------------------------*/

extern void lzh_deleteTemporaryFiles(void);

/*----------------------------------------------------------------------*/

extern FILE *lzh_openFile(const char       *archiveName,
                          LZH_CONTENT_LIST contents,
                          const char       *filename,
                          const char       *mode);

#endif
