/***************************************************************************/
/*                                                                         */
/*                         Error handler for validator                     */
/*                                                                         */
/***************************************************************************/

#include <stdio.h>
#include <setjmp.h>
#include <stdarg.h>
#include <string.h>

#include "BOOL.h"
#include "utilities.h"
#include "readkiss.h"
#include "KISS_data.h"

int errorcount = 0;

jmp_buf read_doll_env;

static FILE *messagefp = NULL;

static char *cnffilename=NULL;

static char messagetext[256];

static BOOL first_error = TRUE;

static char *message_tags[] =
  {
    "CNF0:Illegal configuration data\n",
    "CNF1:Failed to load doll %s\n",
    "CNF2:Failed to open file %s\n",
    "CNF3:Can't read palette file %s\n",
    "CNF3a:Incorrect palette flag byte (%02x rather than %02x)\n",
    "CNF3b:Palette claims to use %1d bits per col (should be 12 or 24)\n",
    "CNF3c:Number of colours per group (%1d) is out of range\n",
    "CNF3d:Number of palette groups (%1d) is out of range\n",
    "CNF3e:Palette file length does not match header\n",
    "CNF3f:Missing palette file %s\n",
    "CNF4:Failed to parse palette data\n",
    "CNF5:<Not used>\n",
    "CNF6:Too many sets\n",
    "CNF7:Missing cel file %s\n",
    "CNF8:Bad header in cel %s\n",
    "CNF8a:Cel claims to use %1d bits per pixel (should be 4 or 8)\n",
    "CNF8b:Cherry cel claims to use %1d bits per pixel (should be 32)\n",
    "CNF8c:Incorrect cel flag byte (%02x rather than %02x or %02x)\n",
    "CNF9:Bad pixel data in %s - file length does not match header\n",
    "CNF10:Bad image size definition - should be (width,height)\n",
    "CNF10b:Bad terminator for image size (')' required)\n",
    "CNF11:Bad cel coordinates for set %1d\n",
    "CNF12:Bad Mark value for cel\n",
    "CNF13:Bad palette number for set %1d\n",
    "CNF14:<Not Used>\n",
    "CNF15:Can't resolve object Mark #%1d in FKISS command\n",
    "CNF16:Can't resolve cel %s in FKISS command\n",
    "CNF17:Bad palette reference\n",
    "CNF17a:No palette available for cel %s\n",
    "CNF17b:Bad palette number for cel %s\n",
    "CNF17c:Colour depth for cel %s does not match palette %s\n",
    "CNF18:Bad \'Fix\' attribute\n",
    "CNF19:Bad set info\n",
    "CNF20:Bad transparency field\n",
    "CNF21:Can't parse cel filename\n",
    "CNF22:Bad filename /%s/\n",
    "CNF23:Orphan \':\' in cel definition\n",
    "CNF24:Cel filename should not be quoted\n",
    "CNF25:Object %1d is present in set %1d but given no position\n",
    "CNF26:Object %1d does not exist in set %1d but has a position\n",
    "CNF27:Non-existent object %1d is given a position in set %1d\n",
    "CNF28:<Unused>",
    "CNF29:Line too long\n",
    "CNF30:Duplicated cel attribute\n",
    "CNF31:Unknown cel attribute \'%%%c\'\n",
    "CNF32:Frame number expected for cel group\n",
    "CNF33:Cel group name expected\n",
    "CNF34:No current cel for cel group\n",
    "CNF35:Missign frame terminator\n",
    "CNF36:Whitespace is not allowed between \';\' and \'%%\' in cel attributes\n",
    "CNF37:\'*\' used for coordinate of non-virtual object #%1d in set %1d\n",
    "CNF38:Bad syntax for \'resolution\' hint : %s\n",
    "CNF39:Non-standard HINT: \'%s\'\n",
    "CNF40:Unknown HINT value for \'%s\' : \'%s\'\n",

    "FKP0:Bad fkiss action in %s\n",
    "FKP1:Bad fkiss event in %s\n",
    "FKP2:Bad parameters in %s\n",
    "FKP3:Internal error - unexpected FKISS action in %s\n",
    "FKP4:Internal error - unexpected FKISS event in %s\n",

    "FKS0:Malformed \'@EventHandler\'\n",
    "FKS1:Bad object in /%s/\n",
    "FKS2:Object expected in /%s/\n",
    "FKS3:Cel expected in /%s/\n",
    "FKS4:Bad cel in /%s/\n",
    "FKS5:Object or cel expected in /%s/\n",
    "FKS6:Bad set number in /%s/\n",
    "FKS7:Bad alarm number in /%s/\n",
    "FKS8:Bad palette number in /%s/\n",
    "FKS9:Bad syntax in /%s/\n",
    "FKS10:Unknown function in /%s/\n",
    "FKS11:Bad filename in /%s/\n",
    "FKS12:Bad integer in /%s/\n",
    "FKS13:Bad parameter list in /%s/\n",
    "FKS14:Bad string in /%s/\n",
    "FKS15:Bad variable or integer in /%s/\n",
    "FKS16:Bad label in /%s/\n",
    "FKS17:Action outside event handler in /%s/\n",
    "FKS18:Missing function\n",
    "FKS19:/%s/ - Missing endif() for \'if\' at line %1d\n",
    "FKS19a:Missing endif() at end of file for \'if\' at line %1d\n",
    "FKS20:else or endif without matching \'if\'\n",
    "FKS21:Illegal \'if\' nesting\n",
    "FKS22:Bad key name\n",
    "FKS23:Duplicated event handler in /%s/\n",
    "FKS24:Filename too long in /%s/\n",
    "FKS25:Ambiguous cel reference in /%s/\n",
    "FKS26:Duplicate \'@EventHandler\'\n",
    "FKS27:%s appears to be FKISS, but no \'@EventHandler\'\n",
    "FKS28:sound(\"\") may not work on all players\n",
    "FKS29:Common misspelling of \'%s\'\n",
    "FKS30:\'%s\' is not an FKISS function - do you mean \'%s\'?\n",
    "FKS31:\'%s\' is not supported by all players\n",
    "FKS32:Effect of \'%s\' varies with player\n",
    "FKS33:Illegal cel group in /%s/\n",
    "FKS34:No such cel group in /%s/\n",
    "FKS35:Event not alllowed in this context : /%s/\n",
    "FKS36:Bad \'tag\' for pool value /%s/\n",

    "FKCase:FKISS function should be lowercase\n",

    "UNREF:Unresolved references or unused handlers may exist :-\n",
    "UNREF01:     Alarm handler at line %1d\n",
    "UNREF02:     Label at line %1d\n",
    "UNREF03:     Timer referenced at line %1d does not have a handler\n",
    "UNREF04:     Label referenced at line %1d does not have a handler\n",

    "BADVAR0:     Identifier \"%s\" is not a label, alarm, or rhs\n",
    "BADVAR1:     Identifier \"%s\" is overloaded\n",
    "BADVAR2:     Identifier \"%s\" is read from, but never written to\n",
    "BADVAR3:     Identifier \"%s\" is written to, but never read from\n",

    "KVER0:KISS/GS1",
    "KVER1:KISS/GS2",
    "KVER2:KISS/GS1+",
    "KVER3:KISS/GS2+",
    "KVER4:FKISS",
    "KVER5:FKISS 2",
    "KVER6:FKISS 3",
    "KVER7:FKISS 4",

    "KCM0:Unknown",
    "KCM1:Original (16 @ 12 bit)",
    "KCM2:Standard (256 @ 24 bit)",
    "KCM3:Extended Palette",
    "KCM4:Cherry KISS",
    "KCM5:Hybrid Cherry/Paletted",

    "STAT0:\nStatistics for %s\n",
    "STAT1:Number of cel references : %1d\n",
    "STAT2:Number of palette files  : %1d\n",
    "STAT3:Number of objects        : %1d\n",
    "STAT4:Objects exist in all sets\n",
    "STAT5:Active sets              :\n",
    "KVER:KISS Version             : %s\n",
    "KCM:Colour model             : %s\n",

    "Success:No errors found in %s\n",
    "ErrFnd:Errors found in %s:\n",

    "ScanLZH:Scanning doll archive...\n",
    "MeltLZH:Extracting files...\n",
    "ReadCNF:Validating %s...\n",
    NULL
  };

/***************************************************************************/

char *message_msgtext(char *tag)
{
  char tagcpy[16], *defaultText;
  char *p;
  int len,idx=0, taglen;

  p = strchr(tag,':');
  if ( p )
  {
    len = (int)p - (int)tag;
    strncpy(tagcpy,tag,len);
    tagcpy[len] = 0;
    tag = tagcpy;
    defaultText = &(p[1]);
  }
  else defaultText = tag;

  taglen = strlen(tag);
  while ( message_tags[idx] )
  {
    p = strchr(message_tags[idx],':');
    len = (int)p - (int)(message_tags[idx]);
    if ( (len==taglen) && (struncmp(message_tags[idx],tag,len) == 0) )
    {
      p++;
      return p;
    }
    idx++;
  }
  return defaultText;
}

/***************************************************************************/

static void _report(char *format,va_list args)
{
  vfprintf(messagefp,format,args);
}

/***************************************************************************/

void message_report(char *format,...)
{
  va_list args;

  va_start(args,format);

  _report(format,args);

  va_end(args);
}

/***************************************************************************/

void message_reportTag(char *tag,...)
{
  va_list args;

  va_start(args,tag);
  _report(message_msgtext(tag),args);
  va_end(args);
}

/***************************************************************************/

void message_error(char *msgid,...)
{
  va_list args;

  if ( first_error && (cnffilename != NULL) )
  {
    first_error = FALSE;
    message_reportTag("ErrFnd",cnffilename);
  }

  if ( cnffilename != NULL )
    message_report("Line %1d:",line_count);

  va_start(args,msgid);
  _report(message_msgtext(msgid),args);
  va_end(args);

  errorcount++;

}

/***************************************************************************/

void message_begin(char *cnf)
{
  cnffilename = strdup(cnf);
  first_error = TRUE;
  errorcount  = 0;
}

/***************************************************************************/

void message_progress(char *format,...)
{
  if ( show_progress )
  {
    va_list args;
    va_start(args,format);
    vfprintf(stdout,format,args); /* Progress report mustn't go to the output file */
    va_end(args);
  }
}

/***************************************************************************/

void message_init(char *filename)
{
  if ( filename )
    messagefp = fopen(filename,"w");

  if ( !messagefp )
    messagefp = stdout;
}
