/***************************************************************************/
/*                                                                         */
/*          Operating system specific elements for kiss validator          */
/*                                                                         */
/***************************************************************************/

#ifndef __OSSPECIFIC_H
#define __OSSPECIFIC_H

#define MaxFileNameLen 256

/***************************************************************************/
/*
 * Operating system constants.
 *
 * DIRSEP     : Separator between directories.
 * FILEXT     : Separator between filename and file type extension.
 * CREATEDIR  : Create directory
 *
 * Various lharc commands. These all perform parameter substitutions as:
 *              %d : output directory
 *              %a : archive name
 *              %f : file (name of file to extract, or target for list)
 *
 * LZHEXTRACT    : Extract specific files.
 * LZHEXTRACTALL : Extract entire contents
 * LZHLIST       : List an archive's contents.
 */
/***************************************************************************/

/*
 * Directory & filename separators for RISCOS
 */

#define DIRSEP        '.'
#define FILEEXT       '/'
#define CREATEDIR     "CDIR %d"
#define DELETEDIR     "REMOVE %d"
#define LZHEXTRACT    "lharc -xqlfw=%d %a %f"
#define LZHEXTRACTALL "lharc -xqlfw=%d %a"
#define LZHLIST       "lharc -l >%f %a"

/*
 * Directory & filename separators for UNIX
 */

/*
#define DIRSEP        '/'
#define FILEEXT       '.'
#define CREATEDIR     "mkdir %d"
#define DELETEDIR     "rmdir %d"
#define LZHEXTRACT    "lharc -xqfw=%d %a %f"
#define LZHEXTRACTALL "lharc -xqfw=%d %a"
#define LZHLIST       "lharc -l >%f %a"
*/

/*
 * Directory & filename separators for Windows
 */

/*
#define DIRSEP        '\\'
#define FILEEXT       '.'
#define CREATEDIR     "mkdir %d"
#define DELETEDIR     "rmdir %d"
#define LZHEXTRACT    "lha 2>NUL e -c1 -m1 %a %d\\ %f"
#define LZHEXTRACTALL "lha 2>NUL e -c1 -m1 %a %d\\"
#define LZHLIST       "lha >%f l %a"
*/

/***************************************************************************/
/*
 * Hardware-related constants.
 */
/***************************************************************************/

/*
 * If your hardware is big-endian, replace the '0' in the line below by a '1'
 */

#define BIGENDIAN 0

#define SWAP_SHORT(x) (((x & 0xff)<<8)+((x>>8)&0xff))
#define SWAP_LONG(x)  ((SWAP_SHORT(x)<<16)+(SWAP_SHORT(x>>16)))

/*
 *  It seems some C compilers aren't ANSI-compliant and fail to
 *  define some key constants, so here they are.
 */

#ifndef NULL
#define NULL 0
#endif
#ifndef SEEK_SET
#define SEEK_SET 0
#endif
#ifndef SEEK_END
#define SEEK_END 2
#endif

#endif

