/***************************************************************************/
/*                                                                         */
/*                   Miscellaneous Utilities for KISS Validator            */
/*                                                                         */
/***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <limits.h>

#include "BOOL.h"

#define SGN(x) ((x)<0?-1:1)

/***************************************************************************/

void *safe_alloc(int size)
{
  void *p = malloc(size);
  if ( !p )
  {
    fputs("Out of memory\n",stderr);
    exit(1);
  }

  return p;
}

/*======================================================================================*/

char *strndup(const char *s, int bufflen)
{
  char *v = safe_alloc(bufflen);

  strncpy(v,s,bufflen-1);
  v[bufflen-1] = 0;

  return v;
}

/*======================================================================================*/

char *strdup(const char *s)
{
  return strndup(s,strlen(s)+1);
}

/*======================================================================================*/

int struncmp(const char *s1,const char *s2,int len)
{
  while ( *s1 && *s2 && (len != 1) && (tolower(*s1) == tolower(*s2)) )
  {
    s1++;
    s2++;
    len--;
  }
  return tolower(*s2)-tolower(*s1);
}

#define strucmp(s1,s2) struncmp(s1,s2,0)

/***************************************************************************/

BOOL convert_to_int(char *buffer, int *val)
{ int sign = 1, digit;
  if ( (*buffer == '-') || (*buffer == '+') )
  { sign = *buffer == '-' ? -1 : 1;
    buffer++;
  }

  if ( !*buffer )
    return FALSE;

  *val = 0;
  while ( *buffer )
  {
    if ( !isdigit(*buffer) ) return FALSE;
    digit = *buffer - '0';
    if ( ((INT_MAX - digit) / 10) < *val )
    {
      *val = INT_MAX;
      break;
    }
    *val = (*val * 10) + digit;
    buffer++;
  }
  if ( sign < 0 )
    *val = -*val;
  return TRUE;
}

/***************************************************************************/

char *lowercase(const char *src,char *buffer,int bufflen)
{ int loop;
  char *p = buffer;

  for ( loop = 0; loop < bufflen; loop++ )
  { *p = tolower(*src);
    if ( !*p ) return buffer;
    p++;
    src++;
  }
  buffer[bufflen-1] = 0;
  return buffer;
}
